@@ -1,5 +1,5 @@
 /*  dctrl-tools - Debian control file inspection tools
-    Copyright © 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+    Copyright © 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
                 Antti-Juhani Kaijanaho
 
     This program is free software; you can redistribute it and/or modify
@@ -139,6 +139,7 @@ static struct argp_option options[] = {
 	{ "mmap",           OPT_MMAP, 0,            0, N_("Attempt mmapping input files") },
 	{ "ignore-parse-errors", OPT_IGN_ERRS, 0,   0, N_("Ignore parse errors") },
         { "pattern",        OPT_PATTERN, N_("PATTERN"), 0, N_("Specify the pattern to search for") },
+	{ "whole-pkg",	    'w', 0,                 0, N_("Do (eregex) matching on whole package names") },
 	{ 0 }
 };
 
@@ -224,7 +225,7 @@ struct arguments {
 struct atom * clone_atom(struct arguments * args)
 {
 	if (args->p.num_atoms >= MAX_ATOMS) {
-		message(L_FATAL, _("predicate is too complex"), 0);
+		message(L_FATAL, 0, _("predicate is too complex"));
 		fail();
 	}
 	int oa = args->p.num_atoms-1;
@@ -235,6 +236,7 @@ struct atom * clone_atom(struct arguments * args)
 	rv->field_inx = atom->field_inx;
 	rv->mode = atom->mode;
 	rv->ignore_case = atom->ignore_case;
+	rv->whole_pkg = atom->whole_pkg;
 	rv->pat = atom->pat;
 	rv->patlen = atom->patlen;
 	struct atom_code * ac = args->atom_code[oa];
@@ -297,11 +299,16 @@ static void leave(struct arguments * args, int paren)
 static void apptok(struct arguments * args, const int tok)
 {
 	debug_message("apptok", 0);
+        if (args->finished) {
+                message(L_FATAL, 0,
+                        _("file names are not allowed within the predicate"));
+                fail();
+        }
 	if (args->in_atom && tok < TOK_ATOM_BASE) {
 		finish_atom(args);
 	}
 	if (args->toks_np >= MAX_TOKS) {
-		message(L_FATAL, _("predicate is too long"), 0);
+		message(L_FATAL, 0, _("predicate is too long"));
 		fail();
 	}
 	args->toks[args->toks_np++] = tok;
@@ -333,7 +340,7 @@ static struct atom * enter_atom(struct arguments * args)
 	}
 	args->in_atom = true;
 	if (args->p.num_atoms >= MAX_ATOMS) {
-		message(L_FATAL, _("predicate is too complex"), 0);
+		message(L_FATAL, 0, _("predicate is too complex"));
 		fail();
 	}
 	APPTOK(args->p.num_atoms + TOK_ATOM_BASE);
@@ -352,6 +359,7 @@ static struct atom * enter_atom(struct arguments * args)
 	rv->field_inx = -1;
 	rv->mode = M_SUBSTR;
 	rv->ignore_case = 0;
+	rv->whole_pkg = 0;
 	rv->pat = 0;
 	rv->patlen = 0;
 	return rv;
@@ -360,7 +368,7 @@ static struct atom * enter_atom(struct arguments * args)
 #define set_mode(nmode) do { \
 	atom = ENTER_ATOM; \
 	if (atom->mode != M_SUBSTR) { \
-		message(L_FATAL, _("inconsistent atom modifiers"), 0); \
+		message(L_FATAL, 0, _("inconsistent atom modifiers")); \
 		fail(); \
 	} \
 	atom->mode = (nmode); \
@@ -419,7 +427,7 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
                         }
                         struct field_attr *fa = fieldtrie_insert(s);
                         if (args->num_show_fields >= MAX_FIELDS) {
-                                message(L_FATAL, _("too many output fields"), 0);
+                                message(L_FATAL, 0, _("too many output fields"));
                                 fail();
                         }
                         args->show_fields[args->num_show_fields] = fa->inx;
@@ -443,7 +451,7 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 		int ll = str2loglevel(arg);
 		if (ll < 0)
 		{
-			message(L_FATAL, _("no such log level"), arg);
+			message(L_FATAL, 0, _("no such log level '%s'"), arg);
 			fail();
 		}
 		set_loglevel(ll);
@@ -537,8 +545,8 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
                 debug_message("parse_opt: pattern", 0);
 		atom = ENTER_ATOM;
 		if (atom->pat != 0) {
-                        message(L_FATAL, _("Multiple patterns for the same "
-                                           "atom are not allowed"), 0);
+                        message(L_FATAL, 0, _("Multiple patterns for the same "
+                                              "atom are not allowed"));
                         fail();
                 }
 		atom->patlen = strlen(arg);
@@ -546,6 +554,12 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 		if (atom->pat == 0) fatal_enomem(0);
 		strcpy((char*)atom->pat, arg);
 		break;
+	case 'w':
+		debug_message("parse_opt: whole-pkg", 0);
+		atom = ENTER_ATOM;
+		atom->whole_pkg = 1;
+		set_mode(M_EREGEX);
+		break;
 	case ARGP_KEY_ARG:
 		debug_message("parse_opt: argument", 0);
 	redo:
@@ -569,7 +583,7 @@ static error_t parse_opt (int key, char * arg, struct argp_state * state)
 		if (args->finished) {
 			char const * s;
 			if (args->num_fnames >= MAX_FNAMES) {
-				message(L_FATAL, _("too many file names"), 0);
+				message(L_FATAL, 0, _("too many file names"));
 				fail();
 			}
 			s = strdup(arg);
@@ -615,6 +629,7 @@ static void dump_args(struct arguments * args)
 		printf("atoms[%zi].field_name = %s\n", i, args->p.atoms[i].field_name);
 		printf("atoms[%zi].mode = %i\n", i, args->p.atoms[i].mode);
 		printf("atoms[%zi].ignore_case = %i\n", i, args->p.atoms[i].ignore_case);
+		printf("atoms[%zi].whole_pkg = %i\n", i, args->p.atoms[i].whole_pkg);
 		printf("atoms[%zi].pat = %s\n", i, args->p.atoms[i].pat);
 	}
 	printf("proglen = %zi\n", args->p.proglen);
@@ -658,26 +673,26 @@ static void unexpected(int tok)
 {
 	switch (tok) {
 	case TOK_EOD:
-		message(L_FATAL, _("unexpected end of predicate"), 0);
+		message(L_FATAL, 0, _("unexpected end of predicate"));
 		fail();
 	case TOK_NOT: 
-		message(L_FATAL, _("unexpected '!' in command line"), 0);
+		message(L_FATAL, 0, _("unexpected '!' in command line"));
 		fail();
 	case TOK_AND:
-		message(L_FATAL, _("unexpected '-a' in command line"), 0);
+		message(L_FATAL, 0, _("unexpected '-a' in command line"));
 		fail();
 	case TOK_OR : 
-		message(L_FATAL, _("unexpected '-o' in command line"), 0);
+		message(L_FATAL, 0, _("unexpected '-o' in command line"));
 		fail();
 	case TOK_LP :
-		message(L_FATAL, _("unexpected '(' in command line"), 0);
+		message(L_FATAL, 0, _("unexpected '(' in command line"));
 		fail();
 	case TOK_RP :
-		message(L_FATAL, _("unexpected ')' in command line"), 0);
+		message(L_FATAL, 0, _("unexpected ')' in command line"));
 		fail();
 	default:
 		assert(tok >=TOK_ATOM_BASE);
-		message(L_FATAL, _("unexpected atom in command line"), 0);
+		message(L_FATAL, 0, _("unexpected atom in command line"));
 		fail();
 	}
 }
@@ -690,7 +705,7 @@ static void parse_prim(struct arguments * args)
 		get_token(args);
 		parse_conj(args);
 		if (get_token(args) != TOK_RP) {
-			message(L_FATAL, _("missing ')' in command line"), 0);
+			message(L_FATAL, 0, _("missing ')' in command line"));
 			fail();
 		}
 		return;
@@ -799,45 +814,43 @@ int main (int argc, char * argv[])
 #endif
 	parse_predicate(&args);
 	if (args.pattern_error) {
-		message(L_FATAL, _("A pattern is mandatory"), 0);
+		message(L_FATAL, 0, _("A pattern is mandatory"));
 		fail();
 	}
 
 	if (debug_optparse) { dump_args(&args); return 0; }
 
 	if (args.p.num_atoms == 0) {
-		message(L_FATAL, _("a predicate is required"), 0);
+		message(L_FATAL, 0, _("a predicate is required"));
 		fail();
 	}
 
 	if (!check_predicate(&args.p)) {
-		message(L_FATAL, _("malformed predicate"), 0);
+		message(L_FATAL, 0, _("malformed predicate"));
 		fail();
 	}
 
 	if (args.short_descr && !args.description_selected) {
 		if (args.num_show_fields >= MAX_FIELDS) {
-			message(L_FATAL, _("too many output fields"), 0);
+			message(L_FATAL, 0, _("too many output fields"));
 			fail();
 		}
-		message(L_INFORMATIONAL,
-			_("Adding \"Description\" to selected output fields because of -d"),
-			0);
+		message(L_INFORMATIONAL, 0,
+			_("Adding \"Description\" to selected output fields because of -d"));
                 SET_SHOW_FIELD(description_attr->application_data, 1);
                 args.show_fields[args.num_show_fields] = description_attr->inx;
 		++args.num_show_fields;
 	}
 
         if (args.invert_show && args.num_show_fields == 0) {
-                message(L_FATAL,
-                        _("-I requires at least one instance of -s"), 0);
+                message(L_FATAL, 0,
+                        _("-I requires at least one instance of -s"));
                 fail();
         }
 
 	if (!args.show_field_name && args.num_show_fields == 0) {
-		message(L_FATAL,
-			_("cannot suppress field names when showing whole paragraphs"),
-			0);
+		message(L_FATAL, 0,
+			_("cannot suppress field names when showing whole paragraphs"));
 		fail();
 	}
 
@@ -894,6 +907,7 @@ int main (int argc, char * argv[])
 				struct fsaf_read_rv r = get_whole_para(&para);
 				fwrite(r.b, 1, r.len, stdout);
 				putchar('\n');
+				putchar('\n');
 				continue;
 			}
                         if (args.invert_show) {
